#R example: preparing your data for inverse probability of censoring weighting (IPCW).
#Willem van der Wal (vanderwalresearch.com), 20-12-2017.
#Background: ipw: An R Package for Inverse Probability Weighting. Van der Wal WM, Geskus RB. Journal of
#Statistical Software. September 2011, Volume 43, Issue 13. https://www.jstatsoft.org/article/view/v043i13.

#Setting: a study with a point exposure (e.g. at baseline of a study) in a longitudinal study,
#with baseline and time varying covariates, and with drop-out over time. Illustrated using simulated data,
#with extensive comments.

#Session preparation (clean)
	rm(list = objects())

#Load data (adjust path!!!!)
	load(file = ".../Example_analysis_R/basdat.rda")
	load(file = ".../Example_analysis_R/longdat.complete.rda")
	load(file = ".../Example_analysis_R/longdat.rda")

#Examine the data
basdat[1:10,]
	#Basdat contains the time-fixed data for each simulated individual (one row for each).
	#Including the following variables:
		 #ID - ID number for each simulated individual.
		 #sex - male/female
		 #A - exposure, e.g. to the dust cloud (0=not exposed, 1 = exposed)
		 #endtime - end of the observed follow-up for each individual (time 0 is 9/11)
		 #outcome - either "event" (an event of interest such as developing COPD), "in follow-up" or "dropout
nrow(basdat) #there are 300 individuals
longdat.complete[1:30,]
	#longdat.complete contains for each individual a simulated longitudinal covariate L, measured each year.
	#It includes the following variables:
		#ID - ID number for each simulated individual.
		#fuptime - follow-up time in years
		#L - the longitudinal covariate
longdat[1:30,]
	#longdat is similar to longdat.complete, but the covariate is not measured each year, and therefore "incomplete".

#Model unadjusted for informative censoring (can be done just with basdat):
#A time-to-event (survival) model for the effect of the exposure A on the event of interest.
	basdat$ind.event <- ifelse(basdat$outcome == "event", 1, 0)#create indicator variable for event in basdat
	with(basdat, table(outcome, ind.event)) #crosstable between indicator variable and outcome as a check
	library(survival) #load necessary library for Surv() and coxph() functions - install if necessary
	mod1 <- coxph(Surv(time = endtime, event = ind.event) ~ A, data = basdat) #Fit Cox proportional hazards model
	summary(mod1) #Summary output for model, hazard ratio is 1.92
	library(Epi) #load library for ci.lin function
	ci.lin(mod1)#ci.lin gives more succinct output
	basdat$ind.event <- NULL #remove indicator variable (not needed anymore)
	
#Prepare the dataset for inverse probability of censoring weighting (IPCW)
	with(basdat, table(endtime, outcome)) #Study the time points at which events and dropout occur
		#Conclusion: use regular intervals 0, 1, 2, ..., 10 for dataset. Very convenient.
		#When the endtimes are at more irregular time points, this convenient situation can be obtained by rounding.
		#If necessary also round the measurement times for the longitudinal covariates.
		#However, rounding is not absolutely necessary, a dataset could also be build using irregular intervals.
	timepoints <- 1:10 #These are the end times of each interval, the starting times are 0, 1, 2, ..., 9
	timepoints #contains the chosen time points
	#Build a basic dataset, with all necessary time points for each individual:
		dat1 <- data.frame(
			ID = rep( sort(unique(basdat$ID)), each = length(timepoints) ),	#the unique patient ID's in basdat are first sorted, and each is repeated according to the number of time points
			tstart = rep( timepoints - 1, times = length(unique(basdat$ID)) ),	#starttime for each interval of follow-up, it's just 0, 1, ..., 9 repeated for each patient
			fuptime = rep( timepoints, times = length(unique(basdat$ID)) )	#endtime for each interval of follow-up, it's just 1, 2, ..., 10 repeated for each patient
			)
	#Merge the baseline data
		dat1 <- merge(dat1, basdat, by = "ID", all.x = T, sort = F)
		dat1[1:30,] #examine the first 30 rows
	#Of course, all time points after the individual endtime have to be removed
		nrow(dat1)
		dat1 <- dat1[dat1$fuptime <= dat1$endtime,]
		nrow(dat1)
		dat1[1:30,]
	#Create indicator variables for dropout and event
		dat1$ind.event <- ifelse(with(dat1, outcome == "event" & fuptime == endtime), 1, 0) #create indicator for event
		dat1$ind.dropout <- ifelse(with(dat1, outcome == "dropout" & fuptime == endtime), 1, 0) #create indicator for dropout
		table(dat1$ind.event)
		table(dat1$ind.dropout)
		table(basdat$outcome) #OK, the numbers of events and dropouts match those in basdat
		
#Survival model unadjusted for informative censoring, fitted on the longitudinal data
#Just as an illustration and check, should give exactly the same estimates and p-values as mod1
	mod2 <- coxph(Surv(time = tstart, time2 = fuptime, event = ind.event) ~ A, data = dat1) #Fit Cox proportional hazards model
	summary(mod1)
	summary(mod2) #yes, estimates and p-values are exactly the same for mod1 and mod2

#Below, I will add the longitudinal covariate L. For illustration purposes, I will do this both for the complete and
#incomplete version. Of course, in practice you will not have both.
#For the incomplete version, I will illustrate two forms of imputation.

#Add the longitudinal covariate, version I: complete data
	longdat.complete[1:30,] #examine the longitudinal measurements data (again)
	#merge the data: paste measurement time to end of interval, for each individual
		nrow(dat1) #check number of rows (before)
		dat1 <- merge(dat1, longdat.complete, by = c("ID", "fuptime"), all.x = T, sort = F) #merge
		nrow(dat1) #check number of rows (after)
	names(dat1)[names(dat1) == "L"] <- "L.complete" #rename the new variable (from "L" to "L.complete")
	summary(dat1$L.complete) #summary, including check for missings (no missings)
	dat1[1:50,] #examine the data after merging
	
#Add the longitudinal covariate, version II: incomplete data, with last value carried forward imputation
	longdat[1:30,] #examine the longitudinal measurements data
	#check how much measurements each patient has (method 1: explained element by element)
		kdat <- split(longdat, longdat$ID)	#this is a list, each element contains only the measurements for one individual, as a dataframe
		kdat[1:4] #these are the first 4 elements of the list
		nrow(kdat[[1]]) #this is how to count the number of rows for the first element
		data.frame(ID = kdat[[1]]$ID[1], nmeas = nrow(kdat[[1]])) #this is  how to make a dataframe with the ID of the first element of the list, as well as the number of rows in that element
		kdat2 <- lapply(kdat, function(dat)data.frame(ID = dat$ID[1], nmeas = nrow(dat))) #this is how to do it for each element, put the results in another list
		nmeas.dat <- do.call("rbind", kdat2) #this is a way to paste it all together into one dataframe
		nmeas.dat #examine the result, this can be pasted to basdat
	#check how much measurements each patient has (method 1: compact syntax)
		nmeas.dat2 <- do.call("rbind", lapply(split(longdat, longdat$ID),
			function(dat)data.frame(ID = dat$ID[1], nmeas = nrow(dat)))) #this produces the same as above
	#check how much measurements each patient has (method 2)
		#in R there's always multiple ways to do things
		nmeas.dat3 <- data.frame(
			ID = unique(longdat$ID),
			nmeas = sapply(split(longdat, longdat$ID), function(dat)nrow(dat)))
	#just to show that nmeas.dat, nmeas.dat2 and nmeas.dat3 are all the same:
		nmeas.dat[1:4,];nmeas.dat2[1:4,];nmeas.dat3[1:4,]
		nmeas.dat == nmeas.dat2;nmeas.dat == nmeas.dat3
	#I will just use nmeas.dat and paste it to basdat
		basdat <- merge(basdat, nmeas.dat, all.x = T) #merge
		basdat$nmeas[is.na(basdat$nmeas)] <- 0 #fill in the missings (they have zero measurements)
		basdat <- basdat[order(basdat$ID),] #sort the data again on ID
		table(basdat$nmeas) #there are 12 persons with no measurements at all, could be problematic
	#merge the data: paste measurement time to end of interval, for each individual
		nrow(dat1) #check number of rows (before)
		dat1 <- merge(dat1, longdat, by = c("ID", "fuptime"), all.x = T, sort = F) #merge
		dat1 <- dat1[order(dat1$ID, dat1$tstart),] #make sure the data is properly sorted
		nrow(dat1) #check number of rows (after)
		summary(dat1$L) #468 NA's, out of 1613 measurements
	#fill in the missings with last value carried forward
		library(zoo) #I'm going to use the function na.locf from the zoo library
		#I will split the data into one separate part for each patient, then perform lvcf within that patient, then paste everything together again ("unsplit")
		#Note that as always, you can examine each element directly from the console
			kdat <- split(dat1$L, dat1$ID)
			kdat2 <- lapply(kdat, function(x)na.locf(x, na.rm = F))
			dat1$L.lvcf <- unsplit(lapply(kdat, function(x)na.locf(x, na.rm = F)), dat1$ID)
			summary(dat1$L.lvcf) #still 104 NA's
		#In this case it is also necessary to perform first value carried forwards
			kdat <- split(dat1$L.lvcf, dat1$ID)
			kdat2 <- lapply(kdat, function(x)na.locf(x, na.rm = F))
			dat1$L.lvcf <- unsplit(lapply(kdat, function(x)na.locf(x, fromLast = T, na.rm = F)), dat1$ID)
			summary(dat1$L.lvcf) #still the 15 NAs for the persons without measurements, deal with this later on
		dat1[1:50,] #examine the result

#Add the longitudinal covariate, version III: incomplete data, imputation from mixed model.
	#See above for the number of missing longitudinal measurements.
	#In thise case, I will impute all needed measurements with a mixed model, fitted on the original measurements.
	#This could be useful when assuming that there's an underlying "true" value that affects dropout.
	#I will fit a mixed model with a random intercept and slope on L, and fixed effects of A and sex
		longdat <- merge(longdat, basdat, by = "ID", all.x = T, sort = F) #paste necessary covariates to longdat
		library(nlme) #I will use the lme() function from the nlme library
		mod.L <- lme(L ~ fuptime + A + sex, random = ~ fuptime | ID, data = longdat) #fit the model, only for patients with at least one measurement
		summary(mod.L) #summary statistics for the mixed model
		dat1$L.imputed <- predict(mod.L, newdata = dat1)
		summary(dat1$L.imputed) #still 15 missings (0.9%)

#Plotting the individual data is very useful to gain more insight.
#Basic individual plots, to illustrate the longitudinal data, for one individual (the first).
	plotdat <- dat1[dat1$ID == 1,]
	plotdat2 <- longdat[longdat$ID == 1,]
	with(plotdat, plot(fuptime, L.complete, las = 1, type = "b", lwd = 3, col = "blue",
		main = "1", xlim = c(0, 10), ylim = c(0, 40)))
	with(plotdat, points(fuptime, L.lvcf, col = "orange", cex = 1.5, pch = 20))
	with(plotdat2, points(fuptime, L, col = "red", cex = 1.5, pch = 22))
	with(plotdat, lines(fuptime, L.imputed, col = "green", lwd = 2))
	legend(x = 0, y = 10, legend = c("Measurements: complete", "Imputed LVCF", "Measurements: incomplete", "Imputed LME"),
		lty = c(1, NA, NA, 1), col = c("blue", "orange", "red", "green"), pch = c(20, 20, 22, NA))
	#If you study how this works you can easily add dropout and the event, e.g. as vertical lines!
#Plots of longitudinal data for all individuals, to PDF (adjust path!)
	pdf(file = "c:/Users/wmvdw/Documents/analyse_vraag_WTCHR_01/Output/plot1.pdf", width = 12, height = 8)
	par(mfrow = c(2, 3))
	for(i in unique(dat1$ID)){
		plotdat <- dat1[dat1$ID == i,]
		plotdat2 <- longdat[longdat$ID == i,]
		with(plotdat, plot(fuptime, L.complete, las = 1, type = "b", lwd = 3, col = "blue",
			main = i, xlim = c(0, 10), ylim = c(0, 40)))
		with(plotdat, points(fuptime, L.lvcf, col = "orange", cex = 1.5, pch = 20))
		with(plotdat2, points(fuptime, L, col = "red", cex = 1.5, pch = 22))
		with(plotdat, lines(fuptime, L.imputed, col = "green", lwd = 2))
		legend(x = 0, y = 10, legend = c("Measurements: complete", "Imputed LVCF", "Measurements: incomplete", "Imputed LME"),
			lty = c(1, NA, NA, 1), col = c("blue", "orange", "red", "green"), pch = c(20, 20, 22, NA))
	}
	graphics.off()	
	#Note: more fancy plots can be obtained using either the ggplot2 or lattice library.

#Deal with the individual without measurement of longitudinal covariate L, in the "incomplete"  version
#Impute as you see fit, or remove.
#For now, I will just use the average of the measurements.
	dat1$L.lvcf[is.na(dat1$L.lvcf)] <- mean(longdat$L)
	dat1$L.imputed[is.na(dat1$L.imputed)] <- mean(longdat$L)
	
#Estimate IPCW weights (illustrated for each version of the longitudinal covariate), using the ipw package.
#Note 1: with only the intercept in the model for the numerator, as a stabilization factor. Modify at will.
#Note 2: exposure A added as a predictor of dropout, similarly to Hernán MA, Brumback B, Robins JM (2000)
#	Marginal structural models to estimate the causal effect of zidovudine on the survival of HIV-positive
#	men. Epidemiology 11:561–570.
#For further explanation of the ipw syntax see https://www.jstatsoft.org/article/view/v043i13 .
	library(ipw)
#Estimate weights for "complete" L
	temp1 <- ipwtm(
	   exposure = ind.dropout,
	   family = "survival",
	   numerator = ~ 1,
	   denominator = ~ sex + A + L.complete,
	   id = ID,
	   tstart = tstart,
	   timevar = fuptime,
	   type = "first",
	   data = dat1)
#Estimate weights for L imputed with lvcf.
	temp2 <- ipwtm(
	   exposure = ind.dropout,
	   family = "survival",
	   numerator = ~ 1,
	   denominator = ~ sex + A + L.lvcf,
	   id = ID,
	   tstart = tstart,
	   timevar = fuptime,
	   type = "first",
	   data = dat1)
#Estimate weights for L imputed with mixed model.
	temp3 <- ipwtm(
	   exposure = ind.dropout,
	   family = "survival",
	   numerator = ~ 1,
	   denominator = ~ sex + A + L.imputed,
	   id = ID,
	   tstart = tstart,
	   timevar = fuptime,
	   type = "first",
	   data = dat1)
#descriptives for the models used to estimate the denominator of the weights
	library(Epi)
	ci.lin(temp1$den.mod)
	ci.lin(temp2$den.mod)
	ci.lin(temp3$den.mod)
#descriptives and plots for the weights
	par(mfrow = c(2, 2))
	ipwplot(weights = temp1$ipw.weights, timevar = dat1$fuptime, las = 1,
		binwidth = 1, ylim = c(-1, 1), main = "Stabilized inverse probability of censoring weights")
	ipwplot(weights = temp2$ipw.weights, timevar = dat1$fuptime, las = 1,
		binwidth = 1, ylim = c(-1, 1), main = "Stabilized inverse probability of censoring weights")
	ipwplot(weights = temp3$ipw.weights, timevar = dat1$fuptime, las = 1,
		binwidth = 1, ylim = c(-1, 1), main = "Stabilized inverse probability of censoring weights")
	summary(temp1$ipw.weights)
	summary(temp2$ipw.weights)
	summary(temp3$ipw.weights)
	
#Fit weighted models (illustrated for each version of the longitudinal covariate)
#With robust (sandwich) SE estimate.
	mod3a <- coxph(Surv(time = tstart, time2 = fuptime, event = ind.event) ~ A + cluster(ID),
		weights = temp1$ipw.weights, data = dat1)
	mod3b <- coxph(Surv(time = tstart, time2 = fuptime, event = ind.event) ~ A + cluster(ID),
		weights = temp2$ipw.weights, data = dat1)
	mod3c <- coxph(Surv(time = tstart, time2 = fuptime, event = ind.event) ~ A + cluster(ID),
		weights = temp3$ipw.weights, data = dat1)
	ci.lin(mod2)
	ci.lin(mod3a)
	ci.lin(mod3b)
	ci.lin(mod3c)
#After correcting for possible informative dropout, the effect changes from 0.65 to about 0.77 or 0.78.
	


